"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const defaultsDeep = require("lodash.defaultsdeep");
const processor_1 = require("./processor");
const parser_1 = require("./parser");
const assembler_1 = require("./assembler");
const style_compiler_1 = require("./style-compiler");
const template_compiler_1 = require("./template-compiler");
const defaultOptions = require("./default-options");
const utils_1 = require("./utils");
function compile(content, options = {}) {
    return __awaiter(this, void 0, void 0, function* () {
        options = defaultsDeep({}, options, defaultOptions[options.mode || 'production'], {
            scopeId: utils_1.genId(options.filename),
        });
        const sourceMapOptions = {
            filename: options.filename,
            sourceMaps: options.sourceMaps,
            sourceRoot: options.sourceRoot,
        };
        const components = parser_1.default(content, Object.assign({}, sourceMapOptions, options.parseOptions));
        const promises = [null, null, null];
        if (components.script) {
            promises[0] = processor_1.default(components.script, options.processOptions);
        }
        if (components.template) {
            promises[1] = processor_1.default(components.template, options.processOptions)
                .then((item) => template_compiler_1.default(item, Object.assign({}, options.templateOptions, { ssrOptimize: options.ssrOptimize })));
        }
        promises[2] = Promise.all(components.styles.map((style) => {
            return processor_1.default(style, options.processOptions)
                .then((item) => style_compiler_1.default(item, Object.assign({ scopeId: options.scopeId }, sourceMapOptions, options.styleOptions)));
        }));
        promises[3] = Promise.all(components.customBlocks.map((block) => {
            return processor_1.default(block, options.processOptions);
        }));
        const [script, template, styles, customBlocks] = yield Promise.all(promises);
        const assembleResult = assembler_1.default({ script, template, styles, customBlocks }, Object.assign({ styleSourceMaps: options.sourceMaps, ssrOptimize: options.ssrOptimize, scopeId: options.scopeId }, sourceMapOptions, options.assembleOptions));
        const result = Object.assign({}, assembleResult, { scopeId: options.scopeId });
        if (template) {
            result.tips = template.tips;
            result.errors = template.errors;
            result.functional = !!template.functional;
        }
        return result;
    });
}
exports.default = compile;
