"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const source_map_1 = require("source-map");
const transpile = require('vue-template-es2015-compiler');
const VueCompiler = require('vue-template-compiler');
function compileTemplate(item, options) {
    const block = Object.assign({}, item, { functional: !!item.attrs.functional });
    if (block.src) {
        return block;
    }
    const method = options.ssrOptimize ? 'ssrCompile' : 'compile';
    const fnArgs = block.functional ? '_h,_vm' : '';
    const result = VueCompiler[method](block.sourceNode.toString(), options.compileOptions);
    if (result.errors && result.errors.length) {
        block.errors = result.errors;
    }
    let code = `({ render: function(${fnArgs}) { ${result.render} }, staticRenderFns: [ `;
    for (const fn of result.staticRenderFns) {
        code += `function(${fnArgs}) { ${fn} }, `;
    }
    code += '] })';
    code = transpile(code, { transforms: { stripWithFunctional: block.functional } });
    if (result.tips && result.tips.length) {
        block.tips = result.tips;
    }
    block.sourceNode = new source_map_1.SourceNode(null, null, options.filename, code);
    return block;
}
exports.default = compileTemplate;
